# Copyright 2018 The GNOME Music developers
#
# GNOME Music is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# GNOME Music is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with GNOME Music; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# The GNOME Music authors hereby grant permission for non-GPL compatible
# GStreamer plugins to be used and distributed together with GStreamer
# and GNOME Music.  This permission is above and beyond the permissions
# granted by the GPL license by which GNOME Music is covered.  If you
# modify this code, you may extend this exception to your version of the
# code, but you are not obligated to do so.  If you do not wish to do so,
# delete this exception statement from your version.

from enum import IntEnum

from gi.repository import GObject

from gnomemusic import log


class Search(GObject.GObject):

    class State(IntEnum):
        """States the Search can have

           NONE: No ongoing search.
           RESULT: Search with result.
           NO_RESULT: Search without result.
        """
        NONE = 0
        RESULT = 1
        NO_RESULT = 2

    search_mode_active = GObject.Property(type=bool, default=False)
    state = GObject.Property(type=int, default=0)

    def __repr__(self):
        return "<Search>"

    @log
    def __init__(self):
        super().__init__()
